
import React, { useState, useRef, useEffect } from 'react';
import { Button, message, Modal, Row, Col, Card, Tree, Input, Tag, Skeleton } from 'antd';
import {
    FooterToolbar,
    PageContainer,
    ProTable,
    LightFilter,
} from '@ant-design/pro-components';
import { PlusOutlined, ExportOutlined, DownOutlined, DeleteOutlined, ExclamationCircleOutlined } from '@ant-design/icons';
import Http from '@/utils/http';
import Edit from './components/edit'
import Pwd from './components/pwd'
import { TextFilter, SelectFilter, DatetimeRangeFilter, LinkFilter } from '@/components/TableSearch'
import { transformTreeData, getTreeAllKeys } from '@/utils/tree-util'
import { statusMap } from '@/utils/enum-util'

const handleSave = async (val) => {
    const hide = message.loading('正在添加');
    try {
        const resp = await Http.doPost('api/user/save', val);
        if (resp.code === 200) {
            message.success('添加成功');
            return true;
        } else {
            message.error(resp.msg);
            return false;
        }
    } catch (error) {
        message.error('添加失败请重试！');
        return false;
    } finally {
        hide();
    }
};

const handleUpdate = async (val) => {
    const hide = message.loading('正在更新');
    try {
        const resp = await Http.doPost('api/user/update', val);
        if (resp.code === 200) {
            message.success('更新成功');
            return true;
        } else {
            message.error(resp.msg);
            return false;
        }
    } catch (error) {
        message.error('配置失败请重试！');
        return false;
    } finally {
        hide();
    }
};

const handleRemove = async (rows) => {
    if (!rows) return true;
    const hide = message.loading('正在删除');
    try {
        const resp = await Http.doPost('api/user/remove/' + rows.map((row) => row.id).join(','));
        if (resp.code === 200) {
            message.success('删除成功');
            return true;
        } else {
            message.error(resp.msg);
            return false;
        }
    } catch (error) {
        message.error('删除失败，请重试');
        return false;
    } finally {
        hide();
    }
};

const handleExport = async () => {
    const hide = message.loading('正在导出');
    try {
        const resp = await Http.doPostBlob('api/user/export');
        const blob = new Blob([resp], { type: 'application/vnd.ms-excel' });
        window.location.href = window.URL.createObjectURL(blob);
        message.success('导出成功');
        return true;
    } catch (error) {
        message.error('导出失败，请重试');
        return false;
    } finally {
        hide();
    }
};

const handlePwd = async (val) => {
    const hide = message.loading('正在重置');
    try {
        const resp = await Http.doPost('api/user/reset/pwd', val);
        if (resp.code === 200) {
            message.success('重置成功');
            return true;
        } else {
            message.error(resp.msg);
            return false;
        }
    } catch (error) {
        message.error('重置失败请重试！');
        return false;
    } finally {
        hide();
    }
};


const Main = () => {
    const formTableRef = useRef();
    const actionRef = useRef();
    const [selectedRows, setSelectedRows] = useState([]);
    const [modalVisible, setModalVisible] = useState(false);
    const [currentRow, setCurrentRow] = useState();
    const [searchParam, setSearchParam] = useState({});

    const [boxHeight, setBoxHeight] = useState();
    const [deptTreeLoading, setDeptTreeLoading] = useState(false);
    const [deptTreeData, setDeptTreeData] = useState([]);
    const [deptExpandedKeys, setDeptExpandedKeys] = useState([]);

    const [pwdModalVisible, setPwdModalVisible] = useState(false);

    const loadDeptTreeData = (param) => {
        setDeptTreeLoading(true);
        Http.doPostPromise('/api/dept/tree', param, (res) => {
            setDeptTreeLoading(false);
            const rootMenu = { id: 0, name: '全部', value: 0, children: [] };
            rootMenu.children = res.data;
            // const treeData = transformTreeData([rootMenu]);
            const treeData = transformTreeData(res.data);
            setDeptTreeData(treeData);
            const treeAllKeys = getTreeAllKeys(treeData);
            setDeptExpandedKeys(treeAllKeys);
        }).catch((err) => {
            setDeptTreeLoading(false);
        })
    }

    useEffect(() => {
        const handleResize = () => setBoxHeight(window.innerHeight - 368);
        window.addEventListener('resize', handleResize);
        handleResize();

        loadDeptTreeData();

        return () => window.removeEventListener('resize', handleResize);
    }, []);

    const columns = [
        {
            title: 'No',
            dataIndex: 'index',
            valueType: 'indexBorder',
            width: 30,
        },
        {
            title: '名称',
            dataIndex: 'nickname',
            valueType: 'text',
            hidden: false,
            width: 100,
            filterDropdown: (props) => <TextFilter
                name='nickname'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '登录账号',
            dataIndex: 'username',
            valueType: 'text',
            hidden: false,
            width: 100,
            filterDropdown: (props) => <TextFilter
                name='username'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '密码',
            dataIndex: 'password',
            valueType: 'password',
            hidden: true,
            width: 100,
            filterDropdown: (props) => <TextFilter
                name='password'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '头像',
            dataIndex: 'avatar',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='avatar'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '工号',
            dataIndex: 'code',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='code'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '性别',
            dataIndex: 'sex$',
            valueType: 'text',
            hidden: false,
            width: 80,
            filterDropdown: (props) => <SelectFilter
                name='sex'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
                data={[
                    { label: '未知', value: 0 },
                    { label: '男', value: 1 },
                    { label: '女', value: 2 },
                ]}
            />,
        },
        {
            title: '手机号',
            dataIndex: 'phone',
            valueType: 'text',
            hidden: false,
            width: 100,
            filterDropdown: (props) => <TextFilter
                name='phone'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '邮箱',
            dataIndex: 'email',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='email'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '邮箱验证',
            dataIndex: 'emailVerified$',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <SelectFilter
                name='emailVerified'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
                data={[
                    { label: '否', value: 0 },
                    { label: '是', value: 1 },
                ]}
            />,
        },
        {
            title: '所属部门',
            dataIndex: 'deptId$',
            valueType: 'text',
            hidden: false,
            width: 100,
            render: (_, record) => {
                if (_ && _ !== '-') {
                    return <Tag color={""}>{_}</Tag>
                } else {
                    return '-'
                }
            },
            // filterDropdown: (props) => <LinkFilter
            //     name='deptId'
            //     major='dept'
            //     {...props}
            //     actionRef={actionRef}
            //     setSearchParam={setSearchParam}
            // />,
        },
        {
            title: '真实姓名',
            dataIndex: 'realName',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='realName'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '身份证号',
            dataIndex: 'idCard',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='idCard'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '出生日期',
            dataIndex: 'birthday',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='birthday'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '个人简介',
            dataIndex: 'introduction',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='introduction'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '状态',
            dataIndex: 'status$',
            valueType: 'text',
            hidden: false,
            width: 80,
            render: (_, record) => {
                const status = statusMap[record.status]
                return <Tag color={status.color}>{status.text}</Tag>
            },
            filterDropdown: (props) => <SelectFilter
                name='status'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
                data={[
                    { label: '正常', value: 1 },
                    { label: '禁用', value: 0 },
                ]}
            />,
        },
        {
            title: '添加时间',
            dataIndex: 'createTime$',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <DatetimeRangeFilter
                name='createTime'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '修改时间',
            dataIndex: 'updateTime$',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <DatetimeRangeFilter
                name='updateTime'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '修改人员',
            dataIndex: 'updateBy$',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <LinkFilter
                name='updateBy'
                major='user'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },
        {
            title: '备注',
            dataIndex: 'memo',
            valueType: 'text',
            hidden: true,
            width: 140,
            filterDropdown: (props) => <TextFilter
                name='memo'
                {...props}
                actionRef={actionRef}
                setSearchParam={setSearchParam}
            />,
        },

        {
            title: '操作',
            dataIndex: 'option',
            width: 150,
            valueType: 'option',
            render: (_, record) => [
                <Button
                    type="link"
                    key="edit"
                    onClick={() => {
                        setModalVisible(true);
                        setCurrentRow(record);
                    }}
                >
                    编辑
                </Button>,
                <Button
                    type="link"
                    key="pwd"
                    onClick={() => {
                        setPwdModalVisible(true);
                        setCurrentRow(record);
                    }}
                >
                    重置密码
                </Button>,
                <Button
                    type="link"
                    danger
                    key="remove"
                    onClick={async () => {
                        Modal.confirm({
                            title: '删除',
                            content: '确定删除该项吗？',
                            onOk: async () => {
                                const success = await handleRemove([record]);
                                if (success) {
                                    if (actionRef.current) {
                                        actionRef.current.reload();
                                    }
                                }
                            },
                        });
                    }}
                >
                    删除
                </Button>,
            ],
        },
    ];

    return (
        <PageContainer
            header={{
                breadcrumb: {},
            }}
        >
            <Row gutter={[16, 24]}>
                <Col lg={6} md={24}>
                    <Card title="部门" style={{ width: '100%', height: 'calc(100vh - 200px)' }}>
                        <Input
                            style={{ marginBottom: 10 }}
                            placeholder="请输入"
                            onChange={(e) => {
                                const { value } = e.target;
                                loadDeptTreeData({
                                    condition: value
                                })
                            }}
                        />
                        <div style={{ height: 'calc(100vh - 350px)', overflowY: 'auto' }}>
                            {deptTreeLoading ? (
                                <Skeleton active />
                            ) : (
                                <Tree
                                    showLine
                                    blockNode
                                    defaultExpandAll    // 异步加载失效
                                    expandedKeys={deptExpandedKeys}
                                    onExpand={setDeptExpandedKeys}
                                    treeData={deptTreeData}
                                    switcherIcon={<DownOutlined />}
                                    onSelect={(selectedKeys, info) => {
                                        const deptId = selectedKeys[0];
                                        setSearchParam(prevState => ({
                                            ...prevState,
                                            deptId: deptId
                                        }));
                                        actionRef.current?.reload();
                                    }}
                                />
                            )}
                        </div>
                    </Card>
                </Col>
                <Col lg={18} md={24}>
                    <ProTable
                        key="user"
                        rowKey="id"
                        actionRef={actionRef}
                        formRef={formTableRef}
                        columns={columns}
                        cardBordered
                        scroll={{ y: boxHeight }}
                        dateFormatter="string"
                        pagination={{ pageSize: 20 }}
                        search={false}
                        toolbar={{
                            search: {
                                onSearch: (value) => {
                                    setSearchParam(prevState => ({
                                        ...prevState,
                                        condition: value
                                    }));
                                    actionRef.current?.reload();
                                },
                            },
                            filter: (
                                <LightFilter
                                    onValuesChange={(val) => {
                                    }}
                                >
                                </LightFilter>
                            ),
                            actions: [
                                <Button
                                    type="primary"
                                    danger
                                    key="removeBatch"
                                    hidden={selectedRows?.length === 0}
                                    onClick={async () => {
                                        Modal.confirm({
                                            title: '是否确认删除所选数据项?',
                                            icon: <ExclamationCircleOutlined />,
                                            content: '请谨慎操作',
                                            async onOk() {
                                                const success = await handleRemove(selectedRows);
                                                if (success) {
                                                    setSelectedRows([]);
                                                    actionRef.current.reload();
                                                }
                                            },
                                            onCancel() { },
                                        });
                                    }}
                                >
                                    <DeleteOutlined />
                                    删除
                                </Button>,
                                <Button
                                    type="primary"
                                    key="save"
                                    onClick={async () => {
                                        setModalVisible(true)
                                    }}
                                >
                                    <PlusOutlined />
                                    添加
                                </Button>,
                                <Button
                                    key="export"
                                    onClick={async () => {
                                        handleExport();
                                    }}
                                >
                                    <ExportOutlined />
                                    导出
                                </Button>,
                            ],
                        }}
                        request={(params, sorter, filter) =>
                            Http.doPostPromise('/api/user/page1', { ...params, ...searchParam }, (res) => {
                                return {
                                    data: res.data.records,
                                    total: res.data.total,
                                    success: true,
                                }
                            })
                        }
                        tableAlertRender={false}
                        rowSelection={{
                            onChange: (ids, rows) => {
                                setSelectedRows(rows);
                            }
                        }}
                        columnsState={{
                            persistenceKey: 'pro-table-user',
                            persistenceType: 'localStorage',
                            defaultValue: {
                                option: { fixed: 'right', disable: true },
                            },
                            onChange(value) {
                            },
                        }}
                    />
                </Col>

                <Edit
                    open={modalVisible}
                    values={currentRow || {}}
                    treeData={deptTreeData}
                    onCancel={
                        () => {
                            setModalVisible(false);
                            setCurrentRow(undefined);
                        }
                    }
                    onSubmit={async (values) => {
                        let ok = false;
                        if (values.id) {
                            ok = await handleUpdate({ ...values })
                        } else {
                            ok = await handleSave({ ...values })
                        }
                        if (ok) {
                            setModalVisible(false);
                            setCurrentRow(undefined);
                            if (actionRef.current) {
                                actionRef.current.reload();
                            }
                        }
                    }
                    }
                />

                <Pwd
                    open={pwdModalVisible}
                    values={currentRow || {}}
                    onCancel={
                        () => {
                            setPwdModalVisible(false);
                            setCurrentRow(undefined);
                        }
                    }
                    onSubmit={async (values) => {
                        let ok = false;
                        if (values.id) {
                            ok = await handlePwd({ ...values })
                        }
                        if (ok) {
                            setPwdModalVisible(false);
                            setCurrentRow(undefined);
                            if (actionRef.current) {
                                actionRef.current.reload();
                            }
                        }
                    }
                    }
                />
            </Row>
        </PageContainer>
    );
};

export default Main;
